#include <cstdlib>
#include <cmath>
#include <ctime>
#include <iostream>

#ifdef _WIN32
#include <windows.h>
#include <sstream>
#endif

#include "Clock.h"
#include "vgalib.h"

using namespace std;

int MAXX;
int MAXY;

extern Uint8 *buffer;
extern SDL_Surface *surface;
extern unsigned offst[1024];

int main(int argc, char *argv[])
{
    int i, j, omoux=0, omouy=0;
    int moux=0, mouy=0;
    Uint8 events=0;

    switch (argc) {
    case 3:
        MAXX = atoi(argv[1]);
        MAXY = atoi(argv[2]);
	MAXX = 16*(MAXX/16);
	MAXY = 16*(MAXY/16);
        break;
    default:
        MAXX = 640;
        MAXY = 480;
        break;
    }

    // Graphics window and palette setup
    init256();
    pal();
    SDL_WM_SetCaption(
	"Click anywhere with the left mouse button and drag - ESC to exit",
	"Click anywhere with the left mouse button and drag - ESC to exit");
    SDL_EnableUNICODE(1);

    Clock watch;
    Uint32 framesDrawn = 0;

    short *y[2];

    y[0] = (short *) malloc(MAXX*MAXY*sizeof(short));
    y[1] = (short *) malloc(MAXX*MAXY*sizeof(short));

    for(i=0; i<MAXY; i++) {
	for(int j=0; j<MAXX; j++) {
	    y[0][MAXX*i + j] = 0;
	    y[1][MAXX*i + j] = 0;
	}
    }
    
    int idx = 0;

    // Main loop
    while (1) {

	SDL_PumpEvents();

	// Abort if ESC hit
	Uint8 *keys = SDL_GetKeyState(NULL);
	if (keys[SDLK_ESCAPE])
	    break;
	    
	// Get mouse position
	events = SDL_GetMouseState(&moux, &mouy);
	if (omoux != moux || mouy != omouy) {
	    omoux = moux;
	    omouy = mouy;
	}

	short *yOldest   = &y[idx][0];
	idx = (idx+1) % 2;
	short *yOld = &y[idx][0];

	for(i=1; i<MAXY-1; i++) {
	    unsigned lineOffset = offst[i];
	    for(j=1; j<MAXX-1; j++) {
		short val = 
		    ( yOld[offst[i-1] + j-1] + yOld[offst[i-1] + j+1] + 
			yOld[offst[i+1] + j-1] + yOld[offst[i+1] + j+1])/2
		    - yOldest[lineOffset + j];
		yOldest[lineOffset + j] = (short) (0.99999*val);
		//buffer[lineOffset + j] = (Uint8) (127.0+127.0*0.99999*val/8192.);
		buffer[lineOffset + j] = (Uint8) (127.0+224.0*0.99999*val/8192.);
	    }
	}

	// If left mouse button pressed...
	if (events & SDL_BUTTON(SDL_BUTTON_LEFT)) {
	    yOldest[offst[mouy] + moux] = 8192;
	}

        showpage();
	framesDrawn++;
	SDL_Delay(10);
    }
#ifdef _WIN32
    stringstream speed;
    speed << "Rendering " << framesDrawn << " frames in " << (watch.readMS()/1000.0) << " seconds. (" << framesDrawn/(watch.readMS()/1000.0) << " fps)\n";
    MessageBoxA(0, (LPCSTR) speed.str().c_str(), (const char *)"Speed of rendering", MB_OK);
#else
    cout << "Rendering " << framesDrawn << " frames in ";
    cout << (watch.readMS()/1000.0) << " seconds. (";
    cout << framesDrawn/(watch.readMS()/1000.0) << " fps)\n";
#endif
    return 0;
}
